@@ -33,6 +33,7 @@ import numpy as np
 import warnings
 import inspect
 from functools import wraps
+import copy
 import logging
 logger = logging.getLogger(__name__)
 
@@ -89,6 +90,28 @@ class Model(Hdf5Exportable):
             if self.lat is not lattice:  # expect the *same instance*!
                 raise ValueError("Model.__init__() called with different lattice instances.")
 
+    def copy(self):
+        """Shallow copy of self."""
+        cp = copy.copy(self)
+        return cp
+
+    def extract_segment(self, first=0, last=None, enlarge=None):
+        """Return a (shallow) copy with extracted segment of MPS.
+
+        Parameters
+        ----------
+        first, last, enlarge : int
+            See :meth:`~tenpy.models.lattice.Lattice.extract_segment`.
+
+        Returns
+        -------
+        cp : :class:`Model`
+            A shallow copy of `self` with MPO and lattice extracted for the segment.
+        """
+        cp = self.copy()
+        cp.lat = self.lat.extract_segment(first, last, enlarge)
+        return cp
+
     def enlarge_mps_unit_cell(self, factor=2):
         """Repeat the unit cell for infinite MPS boundary conditions; in place.
 
@@ -177,7 +200,7 @@ class NearestNeighborModel(Model):
                 break
         else:
             raise ValueError("All H_bond are `None`!")
-        if self.lat.bc_MPS != 'infinite':
+        if self.lat.bc_MPS == 'finite':
             assert self.H_bond[0] is None
         NearestNeighborModel.test_sanity(self)
         # like self.test_sanity(), but use the version defined below even for derived class
@@ -260,6 +283,14 @@ class NearestNeighborModel(Model):
         # else
         return psi.expectation_value(self.H_bond[1:], axes=(['p0', 'p1'], ['p0*', 'p1*']))
 
+    def extract_segment(self, *args, **kwargs):
+        cp = super().extract_segment(*args, **kwargs)
+        first, last = cp.lat.segment_first_last
+        H_bond = self.H_bond
+        L = len(H_bond)
+        cp.H_bond = [H_bond[i % L] for i in range(first, last + 1)]
+        return cp
+
     def enlarge_mps_unit_cell(self, factor=2):
         """Repeat the unit cell for infinite MPS boundary conditions; in place.
 
@@ -483,10 +514,21 @@ class MPOModel(Model):
         MPOModel.test_sanity(self)
         # like self.test_sanity(), but use the version defined below even for derived class
 
+    def copy(self):
+        cp = super().copy()
+        cp.H_MPO = self.H_MPO.copy()
+        return cp
+
     def test_sanity(self):
         if self.H_MPO.sites != self.lat.mps_sites():
             raise ValueError("lattice incompatible with H_MPO.sites")
 
+    def extract_segment(self, *args, **kwargs):
+        cp = super().extract_segment(*args, **kwargs)
+        first, last = cp.lat.segment_first_last
+        cp.H_MPO = self.H_MPO.extract_segment(first, last)
+        return cp
+
     def enlarge_mps_unit_cell(self, factor=2):
         """Repeat the unit cell for infinite MPS boundary conditions; in place.
 
@@ -525,6 +567,7 @@ class MPOModel(Model):
             The sites grouped together.
         """
         grouped_sites = super().group_sites(n, grouped_sites)
+        self.H_MPO = self.H_MPO.copy()
         self.H_MPO.group_sites(n, grouped_sites)
         return grouped_sites
 
@@ -548,7 +591,7 @@ class MPOModel(Model):
         """
         H_MPO = self.H_MPO
         sites = H_MPO.sites
-        finite = H_MPO.finite
+        finite = (H_MPO.bc == 'finite')
         L = H_MPO.L
         Ws = [H_MPO.get_W(i, copy=True) for i in range(L)]
         # Copy of Ws: we set everything to zero, which we take out and add to H_bond, such that
@@ -1477,7 +1520,7 @@ class CouplingModel(Model):
             raise ValueError("Can't `calc_H_bond` with non-empty `exp_decaying_terms`.")
 
         sites = self.lat.mps_sites()
-        finite = (self.lat.bc_MPS != 'infinite')
+        finite = (self.lat.bc_MPS == 'finite')
 
         ct = self.all_coupling_terms()
         ct.remove_zeros(tol_zero)
@@ -1798,7 +1841,6 @@ class CouplingMPOModel(CouplingModel, MPOModel):
                 *not* use "periodic" boundary conditions.
                 (The MPS is still "open", so this will introduce long-range
                 couplings between the first and last sites of the MPS!)
-
         """
         lat = model_params.get('lattice', self.default_lattice)
         if isinstance(lat, str):
@@ -1813,10 +1855,10 @@ class CouplingMPOModel(CouplingModel, MPOModel):
             bc_MPS = model_params.get('bc_MPS', 'finite')
             order = model_params.get('order', 'default')
             sites = self.init_sites(model_params)
-            bc_x = 'periodic' if bc_MPS == 'infinite' else 'open'
+            bc_x = 'open' if bc_MPS == 'finite' else 'periodic'
             bc_x = model_params.get('bc_x', bc_x)
-            if bc_MPS == 'infinite' and bc_x == 'open':
-                raise ValueError("You need to use 'periodic' `bc_x` for infinite systems!")
+            if bc_MPS != 'finite' and bc_x == 'open':
+                raise ValueError("You need to use 'periodic' `bc_x` for infinite/segment systems!")
             if LatticeClass.dim == 1:  # 1D lattice
                 L = model_params.get('L', 2)
                 # 4) lattice
